/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 21 decembre 2006
   derniere modification le 24/02/2008 */

#ifndef SCEMATRIX_H__
#define SCEMATRIX_H__

#include <SCE/utils/SCEVector.h>

/* matrice 4x4 */
typedef float SCE_TMatrix4[16];

extern SCE_TMatrix4 SCE_TmpMatrix4;


#define SCE_Matrix4_Set(ma, a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p){\
    ma[0]  = a; ma[1]  = b; ma[2]  = c; ma[3]  = d;\
    ma[4]  = e; ma[5]  = f; ma[6]  = g; ma[7]  = h;\
    ma[8]  = i; ma[9]  = j; ma[10] = k; ma[11] = l;\
    ma[12] = m; ma[13] = n; ma[14] = o; ma[15] = p;}


int SCE_Init_Matrix (void);

/* charge la matrice d'identite */
void SCE_Matrix4_LoadIdentity (SCE_TMatrix4);

/* copie une matrice */
void SCE_Matrix4_Copy (SCE_TMatrix4, SCE_TMatrix4);

/* multiplie deux matrices et stocke le resultat dans une autre matrice,
   renvoie un pointeur sur cette derniere */
float* SCE_Matrix4_Mul4 (SCE_TMatrix4, SCE_TMatrix4, SCE_TMatrix4);

/* addition / division (fonctions inutiles a premiere vue) */
void SCE_Matrix4_Add (SCE_TMatrix4, SCE_TMatrix4);
void SCE_Matrix4_Sub (SCE_TMatrix4, SCE_TMatrix4);

/* calcule la transposee d'une matrice */
void SCE_Matrix4_Transpose (SCE_TMatrix4);
/* calcule l'inverse d'une matrice */
void SCE_Matrix4_Inverse (SCE_TMatrix4);

/* construit une matrice de translation */
void SCE_Matrix4_Translate (SCE_TMatrix4, float, float, float);
void SCE_Matrix4_Translatev (SCE_TMatrix4, SCE_TVector3);
/* idem mais multiplie le resultat avec la matrice donnee en argument */
void SCE_Matrix4_MulTranslate (SCE_TMatrix4, float, float, float);
void SCE_Matrix4_MulTranslatev (SCE_TMatrix4, SCE_TVector3);

/* construit une matrice de rotation autour d'un axe */
void SCE_Matrix4_RotX (SCE_TMatrix4, float);
void SCE_Matrix4_RotY (SCE_TMatrix4, float);
void SCE_Matrix4_RotZ (SCE_TMatrix4, float);
/* idem mais multiplie le resultat avec la matrice donnee en argument */
void SCE_Matrix4_MulRotX (SCE_TMatrix4, float);
void SCE_Matrix4_MulRotY (SCE_TMatrix4, float);
void SCE_Matrix4_MulRotZ (SCE_TMatrix4, float);

/* construit une matrice de rotation autour d'un axe quelconque */
void SCE_Matrix4_Rotate (SCE_TMatrix4, float, float, float, float);
void SCE_Matrix4_Rotatev (SCE_TMatrix4, float, SCE_TVector3);
/* idem mais multiplie le resultat avec la matrice donnee en argument */
void SCE_Matrix4_MulRotate (SCE_TMatrix4, float, float, float, float);
void SCE_Matrix4_MulRotatev(SCE_TMatrix4, float, SCE_TVector3);

/* construit une matrice de mise a l'echelle */
void SCE_Matrix4_Scale (SCE_TMatrix4, float, float, float);
void SCE_Matrix4_Scalev (SCE_TMatrix4, SCE_TVector3);
/* idem mais multiplie le resultat avec la matrice donnee en argument */
void SCE_Matrix4_MulScale (SCE_TMatrix4, float, float, float);
void SCE_Matrix4_MulScalev (SCE_TMatrix4, SCE_TVector3);

/* multiplie une matrice avec un vecteur */
void SCE_Matrix4_MulV3 (SCE_TMatrix4, SCE_TVector3);
void SCE_Matrix4_MulV4 (SCE_TMatrix4, SCE_TVector4);

void SCE_Matrix4_Projection (SCE_TMatrix4, float, float, float, float);
void SCE_Matrix4_LookAt (SCE_TMatrix4, SCE_TVector3,
                         SCE_TVector3, SCE_TVector3);

#endif /* guard */
